<?php

/**
 * 2007-2022 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2022 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */
class FsfpayPaymentModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
        if (Configuration::get('FSFPAY_LIVE_MODE', null) == false)
            die('Module is not in live mode');

        if (empty(Context::getContext()->customer->id)) {
            return Tools::redirect('index.php?controller=authentication&back=' . urlencode($this->context->link->getModuleLink('fsfpay', 'payment')));
        }

        $apiKey = Configuration::get('FSFPAY_ACCOUNT_API_KEY');
        $apiUsername = Configuration::get('FSFPAY_ACCOUNT_USERNAME');
        $secretKey = Configuration::get('FSFPAY_ACCOUNT_SECRET_KEY');
        $rate = Configuration::get('FSFPAY_COMMISSION') ? Configuration::get('FSFPAY_COMMISSION') : 0;

        // Redirect to home page if card is empty
        $cart = Context::getContext()->cart;
        $currency = Context::getContext()->currency;
        $customer = Context::getContext()->customer;
        $orderTotal = $cart->getOrderTotal();

        // get this page link
        $link = new Link();
        $url = $link->getPageLink('order', true, null, 'step=3');
        if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active) {
            // error message
            $this->errors[] = $this->module->l('Please fill in all of the required fields');
            return Tools::redirect('index.php?controller=order&step=1');
        }

        if (!Tools::isEmpty('action') && Tools::getValue('action') == 'callback') {
            if (Tools::getIsset('requestHash') && Tools::getValue('paymentStatus') && Tools::getValue('paymentStatus') == 'successful') {
                if (hash('sha256', $apiKey . $apiUsername . $secretKey) === Tools::getValue('requestHash')) {
                    DB::getInstance()->insert('fsfpay', [
                        'paymentId' => Tools::getValue('paymentId'),
                        'orderId' => Tools::getValue('orderId'), // You can use orderId to complete the order
                        'amount' => Tools::getValue('amount'),
                        'email' => Tools::getValue('email'),
                        'currency' => Tools::getValue('currency'),
                        'cryptocurrencyAmount' => Tools::getValue('cryptocurrencyAmount'),
                        'testmode' => Tools::getValue('testmode'),
                        'id_cart' => Tools::getValue('id_cart'),
                        'id_customer' => Tools::getValue('id_customer'),
                    ]);
                    die('OK');
                }
            } else {
                die('status not successful');
            }
        }

        if (!Tools::isEmpty('action') && Tools::getValue('action') == 'successRedirect') {
            $this->module->validateOrder(
                $cart->id,
                Configuration::get('FSFPAY_ORDER_STATUS') ? Configuration::get('FSFPAY_ORDER_STATUS') : Configuration::get('PS_OS_PAYMENT'),
                $cart->getOrderTotal(),
                $this->module->displayName,
                "Payment successful",
                array(),
                $currency->id,
                false,
                $customer->secure_key
            );
            $module_id = $this->module->id;
            $order_id = $this->module->currentOrder;
//            $order_id = Order::getOrderByCartId((int)$cart->id);
            return Tools::redirect('index.php?controller=order-confirmation&id_cart=' . $cart->id . '&id_module=' . $module_id . '&id_order=' . $order_id . '&key=' . $customer->secure_key);
        }

        if (Tools::getValue('action') == 'error') {
            return $this->displayError('An error occurred while trying to redirect the customer');
        } else {

            if (!empty(Tools::getValue('cyrptocurrency'))) {
                if (empty(Tools::getValue('cyrptocurrency')))
                    $cryptocurrency = Configuration::get('FSFPAY_CURRENCY', null) ? Configuration::get('FSFPAY_CURRENCY', null) : 'BTC';
                else
                    $cryptocurrency = Tools::getValue('cyrptocurrency');

                $amount = number_format((1 + ((float) $rate / 100)) * $orderTotal, 2, '.', '');
                $ch = curl_init();
                curl_setopt_array($ch, [
                    CURLOPT_URL => 'https://panel.fsfpay.com/api/payment',
                    CURLOPT_HTTPHEADER => ['Content-Type: application/x-www-form-urlencoded'],
                    CURLOPT_POST => true,
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_POSTFIELDS => http_build_query([
                        'apiKey' => $apiKey,
                        'apiUsername' => $apiUsername,
                        'orderId' => rand(11111, 99999),
                        'email' => $customer->email,
                        'amount' => $amount,
                        'currency' => $currency->iso_code,
                        'cryptocurrency' => $cryptocurrency,
                        'testmode' => Configuration::get('FSFPAY_TEST_MODE') ? 1 : 0,
                        'callbackUrl' => $this->context->link->getModuleLink('fsfpay', 'payment', [
                            'action' => 'callback',
                            'id_cart' => $cart->id,
                            'id_customer' => $customer->id,
                        ]),
                        'successRedirectUrl' => $this->context->link->getModuleLink('fsfpay', 'payment', array('action' => 'successRedirect')),
                        'language' => 'en',
                        'requestHash' => hash('sha256', $apiKey . $apiUsername . $secretKey)
                    ]),
                ]);
                $response = curl_exec($ch);
                $response = json_decode($response, true);
            }

            $this->context->smarty->assign(array(
                'cryptocurrencies' => ['BTC' => 'Bitcoin', 'ETH' => 'Ethereum', 'LTC' => 'Lite Coin', 'TRX' => 'TRON', 'DOGE' => 'DOGE Coin', 'BNB' => 'BNB Coin'],
                'defaultCryptocurrency' => Configuration::get('FSFPAY_CURRENCY', null) ? Configuration::get('FSFPAY_CURRENCY', null) : 'BTC',
                'success' => empty($response['success']) ? 'chooseInstall' : ($response['success'] == true ? true : false),
                'message' => isset($response['message']) ? 'Payment Error: ' . $response['message'] : '',
                'payment_url' => !empty($response['scriptUrl']) ? $response['scriptUrl'] : '',
                //                'payment_url' => $response['iframeUrl'],
            ));
            $this->setTemplatex(_PS_VERSION_ >= '1.7' ? 'success.tpl' : 'success_v16.tpl');
        }
    }

    protected function displayError($message, $description = false)
    {
        /*
         * Create the breadcrumb for your ModuleFrontController.
         */
        $this->context->smarty->assign('path', '
			<a href="' . $this->context->link->getPageLink('order', null, null, 'step=3') . '">' . $this->module->l('Payment') . '</a>
			<span class="navigation-pipe">&gt;</span>' . $this->module->l('Error'));

        /*
         * Set error message and description for the template.
         */
        array_push($this->errors, $this->module->l($message), $description);

        return $this->setTemplate('error.tpl');
    }

    public function setTemplatex($t)
    {
        if (version_compare(_PS_VERSION_, '1.7.0', '>=') === true) {
            return $this->setTemplate('module:fsfpay/views/templates/front/' . $t);
        }
        return $this->setTemplate($t);
    }
}
