<?php
/**
* 2007-2022 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2022 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/
class FsfpayValidationModuleFrontController extends ModuleFrontController
{
    /**
     * This class should be use by your Instant Payment
     * Notification system to validate the order remotely
     */
    public function postProcess()
    {
        /*
         * If the module is not active anymore, no need to process anything.
         */
        if ($this->module->active == false) {
            die;
        }

        $cart_id = Tools::getValue('cart_id');
        $secure_key = Tools::getValue('sec_key');
        if (empty($cart_id) || empty($secure_key)) {
            die('Missing parameters');
        }

        // Restore the context from the $cart_id & the $customer_id to process the validation properly.
        Context::getContext()->cart = new Cart((int) $cart_id);
        Context::getContext()->customer = new Customer((int) Context::getContext()->customer->id);
        Context::getContext()->currency = new Currency((int) Context::getContext()->cart->id_currency);
        Context::getContext()->language = new Language((int) Context::getContext()->customer->id_lang);

        /**
         * Since it is an example, we choose sample data,
         * You'll have to get the correct values :)
         */
        $customer_id = Context::getContext()->customer->id;
        $amount = Context::getContext()->cart->getOrderTotal();
        $secure_key = Context::getContext()->customer->secure_key;
        $payment_status = Configuration::get('PS_OS_PAYMENT');
        $message = 'Payment accepted';
        
//        if ($this->isValidOrder() === true) {
//            $payment_status = Configuration::get('PS_OS_PAYMENT');
//            $message = null;
//        } else {
//            $payment_status = Configuration::get('PS_OS_ERROR');
//
//            /**
//             * Add a message to explain why the order has not been validated
//             */
//            $message = $this->module->l('An error occurred while processing payment');
//        }

        $module_name = $this->module->displayName;
        $currency_id = (int) Context::getContext()->currency->id;

        return $this->module->validateOrder($cart_id, $payment_status, $amount, $module_name, $message, array(), $currency_id, false, $secure_key);
    }

    protected function isValidOrder()
    {
        /*
         * Add your checks right there
         */
        return true;
    }
}
