<?php

/**
 * 2007-2022 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2022 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Fsfpay extends PaymentModule
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'fsfpay';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->author = 'fsfpay (info@fsfpay.com)';
        $this->need_instance = 1;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Fsfpay');
        $this->description = $this->l('');

        $this->confirmUninstall = $this->l('');

        $this->limited_countries = array('AD', 'AE', 'AF', 'AG', 'AI', 'AL', 'AM', 'AO', 'AQ', 'AR', 'AS', 'AT', 'AU', 'AW', 'AX', 'AZ', 'BA', 'BB', 'BD', 'BE', 'BF', 'BG', 'BH', 'BI', 'BJ', 'BL', 'BM', 'BN', 'BO', 'BR', 'BS', 'BT', 'BW', 'BY', 'BZ', 'CA', 'CC', 'CD', 'CF', 'CG', 'CH', 'CI', 'CK', 'CL', 'CM', 'CN', 'CO', 'CR', 'CU', 'CV', 'CX', 'CY', 'CZ', 'DE', 'DJ', 'DK', 'DM', 'DO', 'DZ', 'EC', 'EE', 'EG', 'EH', 'ER', 'ES', 'ET', 'FI', 'FJ', 'FK', 'FM', 'FO', 'FR', 'GA', 'GB', 'GD', 'GE', 'GF', 'GG', 'GH', 'GI', 'GL', 'GM', 'GN', 'GP', 'GQ', 'GR', 'GS', 'GT', 'GU', 'GW', 'GY', 'HK', 'HN', 'HR', 'HT', 'HU', 'ID', 'IE', 'IL', 'IM', 'IN', 'IO', 'IQ', 'IR', 'IS', 'IT', 'JE', 'JM', 'JO', 'JP', 'KE', 'KG', 'KH', 'KI', 'KM', 'KN', 'KP', 'KR', 'KW', 'KY', 'KZ', 'LA', 'LB', 'LC', 'LI', 'LK', 'LR', 'LS', 'LT', 'LU', 'LV', 'LY', 'MA', 'MC', 'MD', 'ME', 'MF', 'MG', 'MH', 'MK', 'ML', 'MM', 'MN', 'MO', 'MP', 'MQ', 'MR', 'MS', 'MT', 'MU', 'MV', 'MW', 'MX', 'MY', 'MZ', 'NA', 'NC', 'NE', 'NF', 'NG', 'NI', 'NL', 'NO', 'NP', 'NR', 'NU', 'NZ', 'OM', 'PA', 'PE', 'PF', 'PG', 'PH', 'PK', 'PL', 'PM', 'PN', 'PR', 'PS', 'PT', 'PW', 'PY', 'QA', 'RE', 'RO', 'RS', 'RU', 'RW', 'SA', 'SB', 'SC', 'SD', 'SE', 'SG', 'SI', 'SJ', 'SK', 'SL', 'SM', 'SN', 'SO', 'SR', 'ST', 'SV', 'SY', 'SZ', 'TC', 'TD', 'TF', 'TG', 'TH', 'TJ', 'TK', 'TL', 'TM', 'TN', 'TO', 'TR', 'TT', 'TV', 'TW', 'TZ', 'UA', 'UG', 'US', 'UY', 'UZ', 'VA', 'VC', 'VE', 'VG', 'VI', 'VN', 'VU', 'WF', 'WS', 'YE', 'YT', 'ZA', 'ZM', 'ZW');

        $this->limited_currencies = array('EUR', 'USD', 'TRY', 'RUB');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        if (in_array($iso_code, $this->limited_countries) == false) {
            $this->_errors[] = $this->l('This module is not available in your country');
            return false;
        }

        Configuration::updateValue('FSFPAY_LIVE_MODE', true);
        Configuration::updateValue('FSFPAY_TEST_MODE', true);
        Configuration::updateValue('FSFPAY_CURRENCY', "BTC");
        Configuration::updateValue('FSFPAY_ACCOUNT_USERNAME', null);
        Configuration::updateValue('FSFPAY_ACCOUNT_API_KEY', null);
        Configuration::updateValue('FSFPAY_ACCOUNT_SECRET_KEY', null);
        Configuration::updateValue('FSFPAY_ORDER_STATUS', 2);
        Configuration::updateValue('FSFPAY_COMMISSION', 0.00);

        include(dirname(__FILE__) . '/sql/install.php');

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('payment') &&
            $this->registerHook('paymentReturn') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('actionPaymentConfirmation') &&
            $this->registerHook('displayPayment') &&
            $this->registerHook('displayPaymentReturn');
    }

    public function uninstall()
    {
        Configuration::deleteByName('FSFPAY_LIVE_MODE');
        Configuration::deleteByName('FSFPAY_TEST_MODE');
        Configuration::deleteByName('FSFPAY_CURRENCY');
        Configuration::deleteByName('FSFPAY_ACCOUNT_USERNAME');
        Configuration::deleteByName('FSFPAY_ACCOUNT_API_KEY');
        Configuration::deleteByName('FSFPAY_ACCOUNT_SECRET_KEY');
        Configuration::deleteByName('FSFPAY_ORDER_STATUS');
        Configuration::deleteByName('FSFPAY_COMMISSION');

        include(dirname(__FILE__) . '/sql/uninstall.php');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitFsfpayModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign([
            'module_dir' => $this->_path,
            'general_form' => $this->renderForm(),
            'transaction_form' => $this->renderTransactionForm(),
        ]);

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $output;
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitFsfpayModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    protected function renderTransactionForm()
    {
        return DB::getInstance()->executeS('SELECT * FROM ' . _DB_PREFIX_ . 'fsfpay');
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        $stateLang = DB::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "order_state_lang");
        foreach ($stateLang as $state) {
            $states[] = array(
                'id_option' => $state['id_order_state'],
                'name' => $state['name']
            );
        }
        //        print_r($this->getConfigFormValues());
        //        exit;
        $comission = Configuration::get('FSFPAY_COMMISSION') ? Configuration::get('FSFPAY_COMMISSION') : 0;
        return array(
            'form' => array(
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Module Status'),
                        'name' => 'FSFPAY_LIVE_MODE',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Test mode'),
                        'name' => 'FSFPAY_TEST_MODE',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'col' => 8,
                        'type' => 'html',
                        'name' => 'FSFPAY_COMMISSION',
                        'value' => Configuration::get("FSFPAY_COMMISSION", null),
                        'label' => $this->l('Comission'),
                        'html_content' => '<input type="number" step="0.01" name="FSFPAY_COMMISSION" value="' . $comission . '" style="-webkit-text-size-adjust: 100%;direction: ltr;-webkit-tap-highlight-color: rgba(0, 0, 0, 0);font: inherit;box-sizing: border-box;display: block;width: 100%;height: 39px;padding: 8px 16px;font-size: 12px;line-height: 1.42857;color: #555;background-color: #fff;background-image: none;border: 1px solid #bbcdd2;border-radius: 4px;transition: border-color ease-in-out 0.15s,box-shadow ease-in-out 0.15s,-webkit-box-shadow ease-in-out 0.15s; margin: 4px 0;">'
                    ),
                    array(
                        'col' => 8,
                        'type' => 'text',
                        'name' => 'FSFPAY_ACCOUNT_USERNAME',
                        'label' => $this->l('API Username'),
                    ),
                    array(
                        'col' => 8,
                        'type' => 'text',
                        'name' => 'FSFPAY_ACCOUNT_API_KEY',
                        'label' => $this->l('API Key'),
                    ),
                    array(
                        'col' => 8,
                        'type' => 'text',
                        'name' => 'FSFPAY_ACCOUNT_SECRET_KEY',
                        'label' => $this->l('Secret Key'),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Order Status'),
                        'name' => 'FSFPAY_ORDER_STATUS',
                        'options' => array(
                            'query' => $states,
                            'id' => 'id_option',
                            'name' => 'name'
                        )
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Select Crypto Currency'),
                        'name' => 'FSFPAY_CURRENCY',
                        'options' => array(
                            'query' => array(
                                array(
                                    'id_option' => 'DOGE',
                                    'name' => 'DOGE Coin',
                                ),
                                array(
                                    'id_option' => 'ETH',
                                    'name' => 'Ethereum',
                                ),
                                array(
                                    'id_option' => 'LTC',
                                    'name' => 'Lite Coin',
                                ),
                                array(
                                    'id_option' => 'TRX',
                                    'name' => 'TRON',
                                ),
                                array(
                                    'id_option' => 'BNB',
                                    'name' => 'BNB Coin',
                                ),
                                array(
                                    'id_option' => 'SHIB',
                                    'name' => 'Shiba Inu',
                                ),
                            ),
                            'id' => 'id_option',
                            'name' => 'name',
                            'default' => array(
                                'value' => 'BTC',
                                'label' => 'Bitcoin',
                            )
                        )
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'FSFPAY_LIVE_MODE' => Configuration::get('FSFPAY_LIVE_MODE', null),
            'FSFPAY_TEST_MODE' => Configuration::get('FSFPAY_TEST_MODE', null),
            'FSFPAY_ACCOUNT_USERNAME' => Configuration::get('FSFPAY_ACCOUNT_USERNAME', null),
            'FSFPAY_ACCOUNT_API_KEY' => Configuration::get('FSFPAY_ACCOUNT_API_KEY', null),
            'FSFPAY_ACCOUNT_SECRET_KEY' => Configuration::get('FSFPAY_ACCOUNT_SECRET_KEY', null),
            'FSFPAY_CURRENCY' => Configuration::get('FSFPAY_CURRENCY', null),
            'FSFPAY_ORDER_STATUS' => Configuration::get('FSFPAY_ORDER_STATUS', null),
            'FSFPAY_COMMISSION' => Configuration::get('FSFPAY_COMMISSION', null),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');
    }

    /**
     * This method is used to render the payment button,
     * Take care if the button should be displayed or not.
     */
    public function hookPayment($params)
    {
        print_r($params);
        exit;
        $currency_id = $params['cart']->id_currency;
        $currency = new Currency((int)$currency_id);

        if (in_array($currency->iso_code, $this->limited_currencies) == false)
            return false;

        $this->smarty->assign('module_dir', $this->_path);

        //        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
        return $this->context->smarty->fetch($this->local_path . 'views/templates/hook/payment.tpl');
    }

    /**
     * This hook is used to display the order confirmation page.
     */
    public function hookPaymentReturn($params)
    {
        if ($this->active == false)
            return;

        $order = $params['objOrder'];

        if ($order->getCurrentOrderState()->id != Configuration::get('PS_OS_ERROR'))
            $this->smarty->assign('status', 'ok');

        $this->smarty->assign(array(
            'id_order' => $order->id,
            'reference' => $order->reference,
            'params' => $params,
            'total' => Tools::displayPrice($params['total_to_pay'], $params['currencyObj'], false),
        ));

        return $this->display(__FILE__, 'views/templates/hook/confirmation.tpl');
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (Configuration::get('FSFPAY_LIVE_MODE', true) == false)
            return;
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $option->setCallToActionText($this->l($this->name . ' with payment'))
            ->setAction($this->context->link->getModuleLink($this->name, 'payment', array(), true));

        return [
            $option
        ];
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function hookActionPaymentConfirmation()
    {
    }

    public function hookDisplayPayment()
    {
        if (Configuration::get('FSFPAY_LIVE_MODE', null) == false)
            return;
        $this->context->smarty->assign(array(
            'this_path' => $this->_path,
            'this_path_bw' => $this->_path,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/'
        ));
        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
        /* Place your code here. */
    }

    public function hookDisplayPaymentReturn($params)
    {
        if (Configuration::get('FSFPAY_LIVE_MODE', null) == false)
            return;

        if (_PS_VERSION_ < "1.7") {

            if (!empty($params['objOrder']))
                $order = $params['objOrder'];
            else
                $order = $params['order'];

            if (
                ($order->getCurrentOrderState()->id != Configuration::get('PS_OS_ERROR') && $order->getCurrentOrderState()->id != Configuration::get('PS_OS_CANCELED') && $order->getCurrentOrderState()->id != Configuration::get('PS_OS_REFUND') && $order->getCurrentOrderState()->id != Configuration::get('PS_OS_OUTOFSTOCK') && $order->getCurrentOrderState()->id != Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')) &&
                $order->getCurrentOrderState()->id == Configuration::get('FSFPAY_ORDER_STATUS')
            )
                $this->smarty->assign('status', 'ok');

            $this->context->smarty->assign(array(
                'id_order' => $order->id,
                'reference' => $order->reference,
                'params' => $params,
                'total' => Tools::displayPrice(
                    !empty($params['total_to_pay']) ? $params['total_to_pay'] : $order->total_paid,
                    !empty($params['currencyObj']) ? $params['currencyObj'] : Currency::getCurrency($order->id_currency),
                    false
                ),
                'shop_name' => Configuration::get('PS_SHOP_NAME'),
                'this_path' => $this->_path,
                'this_path_bw' => $this->_path,
                'this_path_ssl' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/',
            ));
            return $this->display(__FILE__, 'views/templates/hook/confirmation.tpl');
        }
        /* Place your code here. */
    }
}
